% La fonction "exo23_q2" utilise la méthode des variables antithétiques
% pour évaluer plus finement l'espérance de P. Je ne commente que les
% parties qui diffèrent de "exo23_q1". Ici "N" est le nombre /total/ de
% simulations demandé, supposé pair.
function exo23_q2(N)
S = 0;
% Attention : "S2" ne contiendra pas les sommes de carrés des simulations,
% mais les sommes des carrés des /sommes de paires de simulations/.
S2 = 0;
logs = [log(800),log(1000),log(1200)];
C = [.2,.1,.1;.1,.2,.1;.1,.1,.2];
R = chol(C);
% Attention : vu qu'on fera /deux/ simulations à chaque fois, il ne faudra
% faire que N/2 passages dans la boucle. Il sera pratique d'appeler "M" ce
% nombre de passages.
M = N/2;
% La boucle de N/2 simulations commence ici.
for i = 1:M
    % Je fais le choix de variables antithétiques suivant : je vais prendre
    % à chaque fois deux valeurs opposées pour le vecteur gaussien centré
    % qui apparait dans la partie droite de la formule (1). Il est en effet
    % clair que si un vecteur G suit la loi N(C), -G suit la même loi. En
    % outre, il est raisonnable de penser que la corrélation entre les prix
    % simulés en utilisant respectivement les simulations G et -G sera
    % négative, puisque plus le prix de X (par exemple) dans une simulation
    % sera petit, plus il sera grand dans la simulation associée. (Ce qui
    % va se passer quand on prendra le minimum est moins clair, mais il
    % parait raisonnable de penser que la corrélation négative subsistera
    % au moins partiellement. De fait, c'est bien le cas : j'ai calculé que
    % le coefficient de corrélation entre ces deux variables était
    % d'environ -0,73). J'appelle "G" la simulation du vecteur gaussien G
    % de loi N(C) que nous utilisons.
    G = randn(1,3) * R;
    % "P1" est le premier prix simulé, à partir de G.
    P1 = min(exp(logs + G));
    % "P2" est le second prix simulé, à partir de -G.
    P2 = min(exp(logs - G));
    % "sommepaire" est la somme des paires de prix simulés.
    sommepaire = P1 + P2;
    % On met à jour S et S2. Attention, je rappelle que un premier temps ce
    % sont les espérances et variance de "sommepaire" que nous souhaitons
    % calculer, pas celles de P !
    S = S + sommepaire;
    S2 = S2 + sommepaire * sommepaire;
end
% On calcule maintenant l'intervalle de confiance /associé à l'espérance de
% sommepaires/ : c'est donc M qui intervient dans les formules, pas N.
m_sommepaires = S / M;
sigma_sommepaires = sqrt(S2 / M - m_sommepaires * m_sommepaires);
erreur_sommepaires = 2 * sigma_sommepaires / sqrt(M);
% Comme on sait que l'espérance de P est exactement la moitié de
% l'espérance de sommepaires, on en déduit l'estimateur et la marge
% d'erreur pour l'espérance de P.
m = m_sommepaires / 2;
erreur = erreur_sommepaires / 2;
disp(['Prix moyen : ',num2str(m),' ± ',num2str(erreur)]);
return
end